#![allow(dead_code)]

mod ast;
mod builtins;
mod bytecode;
mod compiler;
mod lexer;
mod parser;
mod runtime;
mod tokens;

use crate::ast::AstNode;
use crate::compiler::compile_module;
use crate::lexer::Lexer;
use crate::parser::Parser;
use crate::runtime::Runtime;

fn main() {
    let text = r#"
        {
            
            fn myfunc(a, b) {
                return a + b;
            }
            myfunc(1, 2);
            println("hello world");

        };
"#;
    println!("=== text ===\n    {}", text.replace("\n", "    \n"));
    println!("=== tokenizing... ===");
    let lexer = Lexer::new(text);
    println!("=== tokens ===");
    lexer.for_each(|token| println!("    {}", token.fancy_string(text)));
    println!("=== parsing... ===");
    let mut parser = Parser::new(text, Lexer::new(text));
    let expr = parser.parse_expr();
    println!("=== ast ===\n{}", expr.value.fancy_string(0));
    println!("=== compiling... ===");
    let module = match compile_module(expr) {
        Ok(module) => module,
        Err(error) => return println!("{:#?}", error),
    };
    println!("=== module ===\n{}", module.fancy_string());
    println!("=== executing... ===");
    let runtime = Runtime::new(module);
    match runtime.run() {
        Ok(value) => println!("=== result ===\n{:?}", value),
        Err(error) => println!("{:#?}", error),
    }
}
